@@ -37,6 +37,7 @@ struct connman_network {
 	connman_bool_t secondary;
 	connman_bool_t available;
 	connman_bool_t connected;
+	connman_bool_t hidden;
 	connman_uint8_t strength;
 	connman_uint16_t frequency;
 	char *identifier;
@@ -204,6 +205,9 @@ static DBusMessage *set_property(DBusConnection *conn,
 	} else if (g_str_has_prefix(name, "IPv4.") == TRUE) {
 		int err;
 
+		if (network->ipconfig == NULL)
+			return __connman_error_invalid_property(msg);
+
 		err = __connman_ipconfig_set_ipv4(network->ipconfig,
 							name + 5, &value);
 		if (err < 0)
@@ -211,76 +215,12 @@ static DBusMessage *set_property(DBusConnection *conn,
 	} else
 		return __connman_error_invalid_property(msg);
 
-	__connman_storage_save_network(network);
-
-	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
-}
-
-static DBusMessage *do_connect(DBusConnection *conn,
-					DBusMessage *msg, void *data)
-{
-	struct connman_network *network = data;
-	int err;
-
-	DBG("conn %p", conn);
-
-	if (__connman_security_check_privilege(msg,
-					CONNMAN_SECURITY_PRIVILEGE_MODIFY) < 0)
-		return __connman_error_permission_denied(msg);
-
-	if (network->connected == TRUE)
-		return __connman_error_failed(msg, EALREADY);
-
-	if (network->driver && network->driver->connect) {
-		enum connman_device_mode mode;
-
-		mode = connman_device_get_mode(network->device);
-		if (mode == CONNMAN_DEVICE_MODE_NETWORK_SINGLE)
-			__connman_device_disconnect(network->device);
-
-		err = network->driver->connect(network);
-		if (err < 0 && err != -EINPROGRESS)
-			return __connman_error_failed(msg, -err);
-	} else
-		network->connected = TRUE;
-
-	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
-}
-
-static DBusMessage *do_disconnect(DBusConnection *conn,
-					DBusMessage *msg, void *data)
-{
-	struct connman_network *network = data;
-	int err;
-
-	DBG("conn %p", conn);
-
-	if (__connman_security_check_privilege(msg,
-					CONNMAN_SECURITY_PRIVILEGE_MODIFY) < 0)
-		return __connman_error_permission_denied(msg);
-
-	if (network->connected == FALSE)
-		return __connman_error_failed(msg, EINVAL);
-
-	connman_element_unregister_children(&network->element);
-
-	connman_device_set_disconnected(network->device, TRUE);
-
-	if (network->driver && network->driver->disconnect) {
-		err = network->driver->disconnect(network);
-		if (err < 0 && err != -EINPROGRESS)
-			return __connman_error_failed(msg, -err);
-	} else
-		network->connected = FALSE;
-
 	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
 }
 
 static GDBusMethodTable network_methods[] = {
 	{ "GetProperties", "",   "a{sv}", get_properties },
 	{ "SetProperty",   "sv", "",      set_property   },
-	{ "Connect",       "",   "",      do_connect     },
-	{ "Disconnect",    "",   "",      do_disconnect  },
 	{ },
 };
 
@@ -431,7 +371,10 @@ static void network_destruct(struct connman_element *element)
 	g_free(network->address);
 	g_free(network->identifier);
 
-	connman_ipconfig_unref(network->ipconfig);
+	if (network->ipconfig) {
+		connman_ipconfig_unref(network->ipconfig);
+		network->ipconfig = NULL;
+	}
 }
 
 /**
@@ -461,7 +404,12 @@ struct connman_network *connman_network_create(const char *identifier,
 	__connman_element_initialize(&network->element);
 
 	//temp = connman_dbus_encode_string(identifier);
-	temp = g_strdup(identifier);
+	if (identifier == NULL) {
+		temp = g_strdup("hidden");
+		network->hidden = TRUE;
+	} else
+		temp = g_strdup(identifier);
+
 	if (temp == NULL) {
 		g_free(network);
 		return NULL;
@@ -481,13 +429,7 @@ struct connman_network *connman_network_create(const char *identifier,
 
 	network->type       = type;
 	network->secondary  = FALSE;
-	network->identifier = g_strdup(identifier);
-
-	network->ipconfig = connman_ipconfig_create();
-	if (network->ipconfig == NULL) {
-		connman_network_unref(network);
-		return NULL;
-	}
+	network->identifier = g_strdup(temp);
 
 	return network;
 }
@@ -604,15 +546,12 @@ void connman_network_set_group(struct connman_network *network,
 	if (network->secondary == TRUE)
 		return;
 
-	if (g_strcmp0(network->group, group) == 0)
-		return;
-
 	switch (network->type) {
 	case CONNMAN_NETWORK_TYPE_UNKNOWN:
 	case CONNMAN_NETWORK_TYPE_VENDOR:
+		return;
 	case CONNMAN_NETWORK_TYPE_BLUETOOTH_PAN:
 	case CONNMAN_NETWORK_TYPE_BLUETOOTH_DUN:
-		return;
 	case CONNMAN_NETWORK_TYPE_MBM:
 	case CONNMAN_NETWORK_TYPE_HSO:
 	case CONNMAN_NETWORK_TYPE_WIFI:
@@ -620,6 +559,12 @@ void connman_network_set_group(struct connman_network *network,
 		break;
 	}
 
+	if (g_strcmp0(network->group, group) == 0) {
+		if (group != NULL)
+			__connman_profile_update_network(network);
+		return;
+	}
+
 	if (network->group != NULL) {
 		__connman_profile_remove_network(network);
 
@@ -666,7 +611,7 @@ connman_bool_t __connman_network_get_weakness(struct connman_network *network)
 	case CONNMAN_NETWORK_TYPE_WIMAX:
 		break;
 	case CONNMAN_NETWORK_TYPE_WIFI:
-		if (network->strength < 20)
+		if (network->strength > 0 && network->strength < 20)
 			return TRUE;
 		break;
 	}
@@ -702,6 +647,9 @@ int connman_network_set_available(struct connman_network *network,
  */
 connman_bool_t connman_network_get_available(struct connman_network *network)
 {
+	if (network->hidden == TRUE)
+		return TRUE;
+
 	return network->available;
 }
 
@@ -777,6 +725,7 @@ static gboolean set_connected(gpointer user_data)
 		connman_element_unregister_children(&network->element);
 
 		__connman_device_set_network(network->device, NULL);
+		network->hidden = FALSE;
 
 		__connman_device_decrease_connections(network->device);
 
@@ -785,7 +734,8 @@ static gboolean set_connected(gpointer user_data)
 	}
 
 	network->connecting = FALSE;
-	network->associating = FALSE;
+
+	connman_network_set_associating(network, FALSE);
 
 	return FALSE;
 }
@@ -813,11 +763,6 @@ int connman_network_set_connected(struct connman_network *network,
 		__connman_network_disconnect(network);
 	}
 
-	if (connected == TRUE) {
-		network->connecting = FALSE;
-		network->associating = FALSE;
-	}
-
 	if (network->connected == connected)
 		return -EALREADY;
 
@@ -903,6 +848,8 @@ int __connman_network_connect(struct connman_network *network)
 	if (err < 0) {
 		if (err == -EINPROGRESS)
 			connman_network_set_associating(network, TRUE);
+		else
+			network->hidden = FALSE;
 
 		return err;
 	}
@@ -1298,9 +1245,9 @@ static int network_probe(struct connman_element *element)
 	switch (network->type) {
 	case CONNMAN_NETWORK_TYPE_UNKNOWN:
 	case CONNMAN_NETWORK_TYPE_VENDOR:
+		break;
 	case CONNMAN_NETWORK_TYPE_BLUETOOTH_PAN:
 	case CONNMAN_NETWORK_TYPE_BLUETOOTH_DUN:
-		break;
 	case CONNMAN_NETWORK_TYPE_MBM:
 	case CONNMAN_NETWORK_TYPE_HSO:
 	case CONNMAN_NETWORK_TYPE_WIFI:
@@ -1328,9 +1275,9 @@ static void network_remove(struct connman_element *element)
 	switch (network->type) {
 	case CONNMAN_NETWORK_TYPE_UNKNOWN:
 	case CONNMAN_NETWORK_TYPE_VENDOR:
+		break;
 	case CONNMAN_NETWORK_TYPE_BLUETOOTH_PAN:
 	case CONNMAN_NETWORK_TYPE_BLUETOOTH_DUN:
-		break;
 	case CONNMAN_NETWORK_TYPE_MBM:
 	case CONNMAN_NETWORK_TYPE_HSO:
 	case CONNMAN_NETWORK_TYPE_WIFI:
